function C=maxplus(A,B)

%
% This function implements the max-plus product.
%
% Usage:
% >> C = maxplus(A,B)
%
% Arguments:
% - A is a real-valued matrix of size NxM.
% - B is a real-valued matrix of size MxK.
%
% Returns:
% - C is a real-valued matrix of size NxK.
%

% Copyright (c) 2017 Alex S. Santos and Marcos E. Valle.
% University of Campinas
% Institute of Mathematics, Statistics, and Scientific Computing (IMECC)
% www.ime.unicamp.br/~valle/Codes/PAMMs.zip
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included in 
% all copies or substantial portions of the Software.
%
% The Software is provided "as is", without warranty of any kind.
%
% December 2017
%

% Try to use the mex file if it exists;
if exist('maxplusmex')==3
	% if there exist a .mex file;
	C = maxplusmex(A,B);
else
	% if there is no mex file;
	warning('In order to have fast maxplus product, compile the source code maxplusmex.c to a MEX file.');
	[n,m]=size(A);
	[m,k]=size(B);
	C=-inf*ones(n,k);
	for l=1:m
		for j=1:k        
			C(:,j)=max(C(:,j),A(:,l)+B(l,j));
		end
	end
end
